@@ -69,7 +69,15 @@ class CCXX_CLASS_EXPORT ScriptInterp;
 #define SCRIPT_INDEX_SIZE KEYWORD_INDEX_SIZE
 #define	SCRIPT_PREPROCESSOR_OVERRIDE 1
 #define	SCRIPT_EXCLUSIVE_OVERRIDE 1
+#define	SCRIPT_IF_OVERRIDE 1
 #define	SCRIPT_MAP_TABLES 1
+#define	SCRIPT_LOCAL_DEFINE 1
+#define	SCRIPT_TRANSACTION_GROUPING 1
+#define	SCRIPT_SET_READ 1
+#define	SCRIPT_MAX_ARGS	256
+#define	SCRIPT_DATA_SEGMENTS
+#define	SCRIPT_EXTENDED_EXPRESSIONS
+#define	SCRIPT_NAMED_EVENTS
 
 class Script
 {
@@ -78,6 +86,9 @@ protected:
 
 	typedef bool (ScriptInterp::*Method)(void);
 	typedef char *(ScriptCommand::*Check)(Line *line, ScriptImage *img);
+	typedef bool (*Cond)(ScriptInterp *interp, const char *v);
+	typedef long (*Function)(long *args, unsigned prec);
+	typedef char *(*Meta)(ScriptInterp *interp, const char *token);
 
 	enum	SymType
 	{
@@ -91,7 +102,8 @@ protected:
 		TRIGGER,
 		POINTER,
 		REF,
-		CACHE
+		CACHE,
+		ARRAY
 	};
 	typedef enum SymType SymType;
 
@@ -99,7 +111,7 @@ protected:
 	typedef struct _symbol
 	{
 		struct _symbol *next;
-		char *id;
+		const char *id;
 		struct
 		{
 			unsigned size : 16;
@@ -107,11 +119,37 @@ protected:
 			bool system : 1;
 			bool readonly : 1;
 			bool commit : 1;
+			bool large : 1;
 			SymType type : 6;
 		} flags;
 		char data[1];
 	}	Symbol;
 
+	class Test
+	{
+	public:
+		const char *id;
+		Cond handler;
+		Test *next;
+	};
+
+	class Attr
+	{
+	public:
+		const char *id;
+		Meta meta;
+		Attr *next;
+	};
+
+	class Fun
+	{
+	public:
+		const char *id;
+		unsigned args;
+		Function fn;
+		Fun *next;
+	};
+
 	class Line
 	{
 	public:
@@ -132,22 +170,38 @@ protected:
 	class Name
 	{
 	public:
+		class Event
+		{
+		public:
+			Event *next;
+			Line *line;
+			const char *name;
+		}	*events;
+
 		Name *next;
 		Line *first;
 		Line *trap[TRAP_BITS];
-		Line *skip[10];
 		unsigned long mask;
 		char *name;
 		enum Mode
 		{
 			ORIGINAL,
 			COPIED,
-			COPY
+			COPY,
+			DATA
 		} mode;
 		typedef enum Mode Mode;
 		bool access;
 	};
 
+	class Initial
+	{
+	public:
+		const char *name;
+		unsigned size;
+		const char *value;
+	};
+
 	class Define
 	{
 	public:
@@ -165,15 +219,25 @@ protected:
  	 * @author David Sugar <dyfet@ostel.com>
 	 * @short global lock table.
 	 */
-	class Locks : private Mutex
+	class Locks : private ThreadLock, private MemPager
 	{
+	private:
+		typedef struct _lock
+		{
+			struct _lock *next;
+			unsigned count;
+			ScriptInterp *owner;
+			char id[1];
+		}	lck;
+
+		unsigned count;
+		unsigned getIndex(const char *id);
+		lck *hash[KEYWORD_INDEX_SIZE];		
+		
 	public:
-
-		ScriptInterp *locks[MAX_LOCKS];
-
 		void release(ScriptInterp *interp);
-		bool lock(ScriptInterp *interp, unsigned id);
-		bool unlock(ScriptInterp *interp, unsigned id);
+		bool lock(ScriptInterp *interp, const char *id);
+		bool unlock(ScriptInterp *interp, const char *id);
 
 		Locks();
 	};
@@ -388,7 +452,7 @@ protected:
  * @author David Sugar <dyfet@ostel.com>
  * @short Bayonne script keyword binding tables and compiler constants.
  */
-class CCXX_CLASS_EXPORT ScriptCommand : public Keydata, public Mutex, public Script
+class CCXX_CLASS_EXPORT ScriptCommand : public MemPager, public Mutex, public Script
 {
 private:
 	friend class ScriptImage;
@@ -412,7 +476,7 @@ private:
 	int keyword_count;
 	int trap_count;
 
-protected:
+public:
 	/**
 	 * Get the method handler associated with a given keyword.  This
 	 * is used by ScriptImage when compiling.
@@ -586,9 +650,8 @@ protected:
 	 *
 	 * @param keydata entry for constants.
 	 */
-	ScriptCommand(const char *cfgfile);
+	ScriptCommand();
 
-public:
 	virtual	int mapicmp(const char *s1, const char *s2)
 		{return stricmp(s1, s2);};
 
@@ -610,12 +673,15 @@ class CCXX_CLASS_EXPORT ScriptSymbol : public SharedMemPager, public Script
 private:
 	friend class ScriptInterp;
 
-	int symsize;
-	Symbol *index[SYMBOL_INDEX_SIZE];
+	int symsize, symlimit;
+	Symbol *index[SYMBOL_INDEX_SIZE + 1];
 	Symbol *trigger;
 
 	unsigned getIndex(const char *symbol);
 
+protected:
+	bool setArray(const char *sym, const char *source);
+
 public:
 	/**
 	 * Fetch a symbol entry.  Normally this is presumed to be from
@@ -666,6 +732,7 @@ public:
 		{return symsize;};
 
 	ScriptSymbol(int size, int pgsize = 1024);
+	~ScriptSymbol();
 
 	/**
 	 * Fetch an embedded pointer held in a symbol identifier.
@@ -794,6 +861,15 @@ public:
 	 */
 	bool setAlias(const char *symbol, const char *source);
 
+	/**
+	 * Swap an existing symbol, relinks table sets.
+	 *
+	 * @param symbol name to find.
+	 * @param symbol name to swap with.
+	 * @return bool true if successful.
+	 */
+	bool swapSymbol(const char *oldname, const char *newname);
+
 	/**
 	 * Get an alias.
 	 *
@@ -840,7 +916,7 @@ public:
  * @author David Sugar <dyfet@ostel.com>.
  * @short Script compiler image set.
  */
-class CCXX_CLASS_EXPORT ScriptImage : public MemPager, public Script
+class CCXX_CLASS_EXPORT ScriptImage : public Keydata, public Script
 {
 protected:
 	std::ifstream scrSource;
@@ -848,16 +924,24 @@ protected:
 	ScriptCommand *cmds;
 	int refcount;
 	Name *index[SCRIPT_INDEX_SIZE];
-	char buffer[512];
+	char *buffer;
+	unsigned bufsize;
 	char *bp;
 	bool quote;
+	unsigned paren;
 	Mutex duplock;
 
+	class	InitialList : public Initial
+	{
+	public:
+		InitialList *next;
+	}	*ilist;
+
 	friend class ScriptInterp;
 	friend class ScriptModule;
 
 	char *getToken(char **pre = NULL);
-
+		
 	/**
 	 * Get the interpreter method pointer for a given keyword.
 	 *
@@ -874,7 +958,7 @@ protected:
 	 *
 	 * @param cmdset of keyword table object used.
 	 */
-	ScriptImage(ScriptCommand *cmdset);
+	ScriptImage(ScriptCommand *cmdset, const char *symset);
 
 	/**
 	 * Purge and reload the script image workspace.
@@ -944,7 +1028,23 @@ protected:
 	 * @param script token being substituted.
 	 */
 	virtual const char *getDefined(const char *token)
-		{return cmds->getLast(token);};
+		{return getLast(token);};
+
+	/**
+	 * Used by a derived constructor to load an initialization list.
+	 *
+	 * @param initialization list.
+	 */
+	void load(Initial *ilist);
+
+	/**
+	 * Used to load a single initialization list entry.
+	 *
+	 * @param keyword
+	 * @param value
+	 * @param size
+	 */
+	void initial(const char *keyword, const char *value, unsigned size = 0);
 
 public:
 	/**
@@ -996,6 +1096,7 @@ class CCXX_CLASS_EXPORT ScriptInterp : public ScriptSymbol
 private:
 	friend class ScriptImage;
 	friend class Script::Session;
+	friend class Script::Locks;
 	friend class ScriptModule;
 
 #pragma	pack(1)
@@ -1004,12 +1105,17 @@ private:
 	public:
 		Name *script;
 		Line *line, *read;
-		unsigned char index;
+		unsigned short index;
 		ScriptSymbol *local;
-		bool caseflag;
+		bool caseflag : 1;
+		bool tranflag : 1;
+		unsigned decimal : 3;
 	};
 #pragma pack()
 
+	static Attr *attr;
+	static Test *test;
+	static Fun *ifun;
 	static Locks locks;
 	ScriptCommand *cmd;
 	ScriptImage *image;
@@ -1020,7 +1126,6 @@ private:
 	int stack;
 	size_t symsize, pgsize;
 	bool once, loop;
-	char packtoken;
 	unsigned long signalmask;
 
 	bool scrTemplate(void);
@@ -1028,7 +1133,6 @@ private:
 	bool scrDisable(void);
 	bool scrUse(void);
 	bool scrLoadable(void);
-	bool scrPacked(void);
 	bool scrPack(void);
 	bool scrUnpack(void);
 	bool scrOn(void);
@@ -1055,10 +1159,13 @@ private:
 	bool scrList(void);
 	bool scrArm(void);
 	bool scrDisarm(void);
+	bool scrNumber(void);
+	bool scrDecimal(void);
 	bool scrSet(void);
 	bool scrAlias(void);
 	bool scrRef(void);
 	bool scrConst(void);
+	bool scrVar(void);
 	bool scrSize(void);
 	bool scrInit(void);
 	bool scrClear(void);
@@ -1069,16 +1176,18 @@ private:
 	bool scrCase(void);
 	bool scrEndcase(void);
 	bool scrError(void);
-	bool scrIf(void);
 	bool scrIfThen(void);
 	bool scrThen(void);
 	bool scrElse(void);
 	bool scrEndif(void);
+	bool scrBegin(void);
+	bool scrEnd(void);
 	bool scrFor(void);
 	bool scrRead(void);
 	bool scrMap(void);
 	bool scrRepeat(void);
 	bool scrForeach(void);
+	bool scrFordata(void);
 	bool scrTryeach(void);
 	bool scrSwap(void);
 	bool scrDo(void);
@@ -1094,9 +1203,22 @@ private:
 	bool scrSkip(void);
 	bool expConditional(void);
 
+public:
+	static long getRealValue(double val, unsigned prec);
+	long getIntValue(const char *text, unsigned prec);
+	int getExpression(long *list, int max, unsigned prec);
+	static double getDouble(long value, unsigned prec);
+	static long getInteger(long value, unsigned prec);
+	static long getTens(unsigned prec);
+	
+	friend void addFunction(const char *name, unsigned count, Function i);
+	friend void addConditional(const char *name, Cond test);
+	friend void addAttribute(const char *name, Meta meta);
 	friend class ScriptCommand;
 
 protected:
+	unsigned char lckcount;
+
 	/**
 	 * Derive a new instance of the system script interpreter.
 	 *
@@ -1188,6 +1310,16 @@ protected:
 	 */
 	bool scrGoto(void);
 
+	/**
+	 * override if support.
+	 */
+	bool scrIf(void);
+
+	/**
+	 * If version of goto.
+	 */
+	bool ifGoto(void);
+
 	/**
 	 * Method used to encode "data" lines.
 	 */
@@ -1212,6 +1344,25 @@ protected:
 		{return false;};
 
 public:
+	/**
+	 * Set data read pointer to a known data source script.
+	 *
+	 * @param name of script.
+	 * @return true on success.
+	 */
+	bool setData(const char *scrname);
+
+	/**
+	 * Get pack token
+	 */
+	char getPackToken(void);
+
+	/**
+	 * Clear transaction processing group.
+	 */
+	void clrTransactions(void)
+		{script[stack].tranflag = false;};
+
 	/**
 	 * Fetch a variable (symbol) that will be used to store data.
 	 * This advances the index.
@@ -1233,8 +1384,6 @@ public:
 	 */
 	void setTemp(const char *value);
 
-
-protected:
 	/**
 	 * May be used to override processing of indirect content
 	 * requests.  This is used in the Bayonne IVR to fetch
@@ -1246,6 +1395,7 @@ protected:
 	virtual Symbol *getIndirect(char *sym)
 		{return NULL;};
 
+protected:
 	/**
 	 * Advance program to the next script statement.
 	 */
@@ -1301,6 +1451,17 @@ protected:
 	 */
 	bool signal(const char *trapname);
 
+	/**
+	 * Events reference to named @event handlers which have been
+	 * attached to a script.  This allows low level applications
+	 * to invoke an event handler much the way a signal handler
+	 * occurs.
+	 *
+	 * @return true if event handler exists.
+	 * @param name of event handler.
+	 */
+	bool event(const char *evtname);
+
 	/**
 	 * Signals can be referenced by numeric id as well as by symbolic
 	 * name.
@@ -1463,6 +1624,21 @@ public:
 	 */
 	char *getValue(const char *def = NULL);
 
+	/**
+	 * Fetch and concatenate a temporary string expression until
+	 * either a comma or closing parenthesis.
+	 *
+	 * @return pointer to temp string buffer.
+	 */
+	char *getString(void);
+
+	/**
+	 * Fetch a temp workspace buffer.
+	 *
+	 * @return workspace pointer.
+	 */
+	char *getTempBuffer(void);
+
 	/**
 	 * Evaluate the content of an option retrieved with getOption.
 	 *
@@ -1479,6 +1655,14 @@ public:
 	inline Line *getScript(void)
 		{return script[stack].line;};
 
+	/**
+	 * Return if the script has generic event handlers...
+	 *
+	 * @return true if generic handlers.
+	 */
+	inline bool hasEvents(void)
+		{return (script[stack].script->events != NULL);};
+
 	/**
 	 * Return if the current line requires prescanning.
 	 *
@@ -1523,6 +1707,14 @@ public:
 
 	inline int mapnicmp(const char *s1, const char *s2, size_t n)
 		{return cmd->mapnicmp(s1, s2, n);};
+
+	/**
+	 * Get decimal.
+	 *
+	 * @return decimal value.
+	 */
+	inline unsigned getDecimal(void)
+		{return script[stack].decimal;};
 };
 
 #ifdef	CCXX_NAMESPACES
